<?php
/**
 * @file
 * Contains support form for miniOrange 2FA Login Module.
 */
namespace Drupal\miniorange_2fa\Form;

use Drupal\Core\Form\FormBase;
use Drupal\miniorange_2fa\MoAuthUtilities;

 /* Showing LoginSetting form info. */

class MoAuthLoginSettings extends FormBase
{    public function getFormId() {
        return 'miniorange_2fa_login_settings';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {
        global $base_url ;
        $url = $base_url .'/admin/config/people/miniorange_2fa/licensing';

        $form['markup_top_2'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'
        );

        $disabled = False;
        if (!MoAuthUtilities::isCustomerRegistered()) {
            $form['header']  = array(
                '#markup' => '<div class="mo_saml_register_message"><p>You need to <a href="'.$base_url.'/admin/config/people/miniorange_2fa/customer_setup">Register/Login</a> with miniOrange before using this module.</p></div><br>',
            );
            $disabled = True;
        }

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "miniorange_2fa/miniorange_2fa.admin",
                    "miniorange_2fa/miniorange_2fa.license",
                )
            ),
        );

        $form['mo_auth_enable_two_factor'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enable_two_factor'),
            '#prefix' => '<div class="mo2f-setup-header mo2f-setup-header-top-left">Enable Two-Factor module</div><div class="mo2f-info">',
            '#suffix' => '</div><br>',
            '#title' => t('Enable Two-Factor plugin. <span style="color:red;">(If you disable this checkbox, Second-Factor authentication will not be invoked for any user during login.)</span>'),
            '#disabled' => $disabled,
        );

        $license_type = (\Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type') == '')? 'DEMO' : \Drupal::config('miniorange_2fa.settings')->get('mo_auth_2fa_license_type');
        $is_free = $license_type == 'DRUPAL_2FA_PLUGIN' || $license_type == 'PREMIUM' || $license_type == 'DRUPAL8_2FA_MODULE' ? FALSE : TRUE;
        $is_free_class = $is_free ? " mo2f-grayed-out" : "";

        $form['mo_auth_enforce_inline_registration'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_enforce_inline_registration'),
            '#prefix' => '<div class="mo2f-setup-header mo2f-setup-header-top-left">Invoke Inline Registration to setup 2nd factor for users <a href= "'.$url.'"><b>[PREMIUM]</b></a></span></div><div class="mo2f-info ' . $is_free_class . '">',
            '#suffix' => '</div><br>',
            '#disabled' => $is_free ,
            '#title' => t('Enforce 2 Factor registration for users at login time.'),
        );
        $form['markup_idp_login_note'] = array(
            '#markup' => '<div class="mo_saml_highlight_background_note"><b>Note: </b>By default 2nd Factor is enabled after password authentication.
             If you do not want to remember passwords anymore and just login with 2nd Factor, please select below option.</div></br>',
     );

        $form['mo_auth_two_factor_instead_password'] = array(
            '#type' => 'checkbox',
            '#default_value' => \Drupal::config('miniorange_2fa.settings')->get('mo_auth_use_only_2nd_factor'),
            '#prefix' => '<div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left">Select Login Screen Options <a href= "'.$url.'"><b>[PREMIUM]</b></a></div></div><div class="mo2f-info ' . $is_free_class . '"><div class="mo2f-info">',
            '#suffix' => '</div></div>',
            '#disabled' => $is_free,
            '#title' => t('Login with 2nd Factor only (No password required.)'),
            '#description' => '<strong>Note:</strong> To use this feature make sure you have enabled the "ENFORCE 2 FACTOR REGISTRATION FOR USERS AT LOGIN TIME" feature.',
        );
        $form['Submit_LoginSettings_form'] = array(
            '#type' => 'submit',
            '#value' => t('Save Settings'),
            '#disabled' => $disabled,
            '#suffix' => '<br><br><br><br><br><br><br></div>'
        );

        MoAuthUtilities::AddsupportTab( $form, $form_state);

        return $form;
    }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $two_factor           = $form['mo_auth_enable_two_factor']['#value'];
        $inline_registration  = $form['mo_auth_enforce_inline_registration']['#value'];
        $use_only_2nd_factor  = $form['mo_auth_two_factor_instead_password']['#value'];

        $enforce_only_2nd_factor   = $use_only_2nd_factor == 1 ? TRUE : FALSE;
        $enable2Factor             = $two_factor == 1 ? TRUE : FALSE;
        $enforceInlineRegistration = ($inline_registration == 1 || $enforce_only_2nd_factor) ? TRUE : FALSE;

        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_use_only_2nd_factor', $enforce_only_2nd_factor)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_enable_two_factor', $enable2Factor)->save();
        \Drupal::configFactory()->getEditable('miniorange_2fa.settings')->set('mo_auth_enforce_inline_registration', $enforceInlineRegistration)->save();
         drupal_flush_all_caches();
        \Drupal::messenger()->addMessage(t("Login settings updated."), 'status');
    }
}
