<?php

namespace Drupal\miniorange_2fa\Form;

/**
 * @file
 * Email verification functions.
 */
use Drupal\user\Entity\User;
use Drupal\Core\Form\FormBase;
use Drupal\miniorange_2fa\MiniorangeUser;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\miniorange_2fa\AuthenticationType;
use Drupal\miniorange_2fa\AuthenticationAPIHandler;
use Drupal\miniorange_2fa\MiniorangeCustomerProfile;

/**
 * Menu callback for email verification.
 */
class test_kba_authentication extends FormBase
{
    public function getFormId() {
        return 'miniorange_kba_autentication';
    }
    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
    {
        $form['markup_top_2'] = array(
            '#markup' => '<div class="mo_saml_table_layout_1"><div class="mo_saml_table_layout mo_saml_container">'
        );
        $form['markup_library'] = array(
            '#attached' => array(
                'library' => 'miniorange_2fa/miniorange_2fa.admin',
                'library' => 'miniorange_2fa/miniorange_2fa.license',
        ),);

        global $base_url;
        $url = $base_url . '/admin/config/people/miniorange_2fa/setup_twofactor';
        $user = User::load(\Drupal::currentUser()->id());
        $user_id = $user->id();
        $utilities = new MoAuthUtilities();
        $custom_attribute = $utilities::get_users_custom_attribute($user_id);

        $user_email = $custom_attribute[0]->miniorange_registered_email;;
        $input = $form_state->getUserInput();
        $questions = NULL;
        $count = 0;
        $txId = '';


        if (array_key_exists('txId', $input) === FALSE) {
            $customer = new MiniorangeCustomerProfile();
            $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $user_email, NULL, NULL, AuthenticationType::$KBA['code']);
            $auth_api_handler = new AuthenticationAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
            $response = $auth_api_handler->challenge($miniorange_user);
            if ($response->status == 'SUCCESS') {
                $questions = $response->questions;
                $count = count($response->questions);
                $txId = $response->txId;
            } else {
                $message = 'An error occured while processing your request. Please try again.';
                MoAuthUtilities::show_error_or_success_message($message , 'error');
            }
        } else {
            $count = $input['question_count'];
            $questions = array();

            for ($i = 1; $i <= $count; $i++) {
                $ques = isset($input['mo2f_kbaquestion' . $i]) ? $input['mo2f_kbaquestion' . $i] : '';
                $ans = isset($input['mo2f_kbaanswer' . $i]) ? $input['mo2f_kbaanswer' . $i] : '';
                $qa=Array();
                $qa =Array('question'=>$ques,'answer'=>$ans);
                array_push($questions, (object)$qa);



            }
        }
        $form['header']['#markup'] = t('<div class="mo2f-setup-header"><div class="mo2f-setup-header-top-left">Test Security Questions (KBA)</div></div><div class="mo2f-info">Please answer the following questions:</div>');

        $i = 0;
        foreach ($questions as $ques) {
            $i++;
            $form['mo2f_kbaanswer'.$i] = array(
                '#type' => 'textfield',
                '#title' => t($i. '. ' . $ques->question),
                '#size' => 49,
                '#suffix' => '<br>',
                '#attributes' => array(
                    'placeholder' => t('Enter your answer here'),
                ),
            );
            $form['mo2f_kbaquestion'.$i] = array(
                '#type' => 'hidden',
                '#value' => $ques->question,
            );
        }

        $form['txId'] = array(
            '#type' => 'hidden',
            '#value' => $txId,
        );
        $form['question_count'] = array(
            '#type' => 'hidden',
            '#value' => $count,
        );

        $form['actions']['submit'] = array(
            '#type' => 'submit',
            '#value' => t('Verify'),
        );
        $form['actions']['cancel'] = array(
            '#markup' => '<span>&nbsp;&nbsp;&nbsp;</span><a href=" ' .$url. ' ">Cancel Test</a>',
        );

        $form['main_layout_div_end'] = array(
            '#markup' => '<br><br><br><br><br><br><br><br><br><br><br><br></div>',
        );

        MoAuthUtilities::AddsupportTab( $form, $form_state);

        return $form;
    }

    /**
     * Form submit handler for email verify.
     */
    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {

        $form_state->setRebuild();
        $input = $form_state->getUserInput();
        $txId =$input['txId'];

        $user = User::load(\Drupal::currentUser()->id());
        $user_id = $user->id();
        $utilities = new MoAuthUtilities();
        $custom_attribute = $utilities::get_users_custom_attribute($user_id);

        $user_email = $custom_attribute[0]->miniorange_registered_email;
        $count = $input['question_count'];
        $kba = array();
        for ($i = 1; $i <= $count; $i++) {
            $ques = $input['mo2f_kbaquestion' . $i];
            $ans = $input['mo2f_kbaanswer' . $i];
            if(empty($ans)){
                \Drupal::messenger()->addMessage(t('Please enter all the answers first.'), 'error');
                return;
            }
            $qa = array(
                "question" => $ques,
                "answer" => $ans,
            );
            array_push($kba, $qa);
        }

        if (count($kba) > 0) {
            $customer = new MiniorangeCustomerProfile();
            $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $user_email, NULL, NULL, AuthenticationType::$KBA['code']);
            $auth_api_handler = new AuthenticationAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
            $response = $auth_api_handler->validate($miniorange_user, $txId, NULL, $kba);
            // Clear all the messages
          \Drupal::messenger()->deleteAll();

            // read API response
            if ($response->status == 'SUCCESS') {
                $message = 'You have successfully completed the test.';
                MoAuthUtilities::show_error_or_success_message($message , 'status');
            } elseif ($response->status == 'FAILED') {
                \Drupal::messenger()->addMessage(t('The answers you have entered are incorrect. Please try again.'), 'error');
                return;
            }
        }else {
            $message = 'An error occured while processing your request. Please try again.';
            MoAuthUtilities::show_error_or_success_message($message , 'error');
        }
    }
}
